import 'package:twipe_flutter_core/utils/controller/collection_controller.dart';
import 'package:twipe_flutter_core/utils/controller/watch_controller.dart';

class ControllerManager {
  static Map<String, dynamic> _collectionController = {};
  static Map<String, dynamic> _watchController = {};
  static Future<bool>
      registerCollectionController<T extends CollectionController>(
          T controller) async {
    _collectionController[controller.getId()] = controller;
    bool result = await controller.setup();
    if (!result) {
      _collectionController.remove(controller.getId());
    }
    return result;
  }

  static Future<void> registerWatchController<T extends WatchController>(
      T controller) async {
    _collectionController[controller.watcherId] = controller;
  }

  static void unregisterCollectionController(String controllerId) {
    if (_collectionController.containsKey(controllerId)) {
      _collectionController.remove(controllerId);
    }
  }

  static void unregisterWatchController(String controllerId) {
    if (_watchController.containsKey(controllerId)) {
      _watchController.remove(controllerId);
    }
  }

  static T getCollectionController<T extends CollectionController>(
      String controllerId) {
    return _collectionController[controllerId] as T;
  }

  static T getWatchController<T extends WatchController>(String controllerId) {
    return _watchController[controllerId] as T;
  }
}
