/// Test Helper Class
import 'package:flutter_test/flutter_test.dart';

abstract class Tester {
  Map<String, dynamic> get tests;

  Future<void> doTest() async {
    for (dynamic t in tests.keys) {
      print('Testing: ' + t);
      bool ready = await setup();
      expect(ready, true);
      if (!ready) {
        print('Error while setting up: ' + t);
        return;
      }
      await tests[t]();
      bool finish = await clear();
      expect(finish, true);
      if (!finish) {
        print('Error while finishing: ' + t);
        return;
      }
    }
  }

  /// Use This Method to predefine setup before all tests.
  Future<bool> setup();

  /// Use This Method to clear all parameters
  Future<bool> clear();
}
