import 'package:flutter_test/flutter_test.dart';
import 'package:twipe_flutter_core/utils/controller/collection_controller.dart';
import 'package:twipe_flutter_core/utils/controller/controller_manager.dart';
import 'package:twipe_flutter_core/utils/test/tester.dart';
import 'package:twipe_flutter_core/utils/eloquent/model/model.dart';

class CollectionTester extends Tester {
  static const String collectionId = 'collection_tester_collection';

  @override
  Future<bool> setup() async {
    return await ControllerManager.registerCollectionController(
        new _TestCollection(collectionId));
  }

  @override
  Future<bool> clear() async {
    return await ControllerManager.getCollectionController(collectionId)
        .clear();
  }

  @override
  Map<String, dynamic> get tests => {
        'collectionCanAddModel': collectionCanAddModel,
        'collectionCanRemoveModel': collectionCanRemoveModel,
      };

  Future<void> collectionCanAddModel() async {
    _TestModel model =
        ControllerManager.getCollectionController<_TestCollection>(collectionId)
            .createModel({'id': 'test'})!;
    await ControllerManager.getCollectionController(collectionId)
        .saveModel(model);
    expect(
        ControllerManager.getCollectionController(collectionId)
            .hasModel('test'),
        true);
  }

  Future<void> collectionCanRemoveModel() async {
    _TestModel model =
        ControllerManager.getCollectionController<_TestCollection>(collectionId)
            .createModel({'id': 'test'})!;
    await ControllerManager.getCollectionController(collectionId)
        .saveModel(model);
    expect(
        ControllerManager.getCollectionController(collectionId)
            .hasModel('test'),
        true);
    await ControllerManager.getCollectionController(collectionId)
        .removeModelById(model.getId());
    expect(
        ControllerManager.getCollectionController(collectionId)
            .hasModel('test'),
        false);
  }
}

///HELPER

class _TestCollection extends CollectionController<_TestModel> {
  _TestCollection(String id) : super(id);

  @override
  _TestModel? createModel(Map<String, dynamic> modelData) {
    _TestModel collectionHelperModel = _TestModel(modelData);
    if (!collectionHelperModel.validate()) {
      return null;
    }
    return collectionHelperModel;
  }
}

class _TestModel extends Model {
  _TestModel(Map<String, dynamic> data) : super(data);

  @override
  List<String> get baseFields => ['id=required'];

  @override
  List<String> get hiddenFields => [];

  @override
  List<String> get validator => [];

  @override
  String get defaultCollection => CollectionTester.collectionId;
}
