import 'package:flutter_test/flutter_test.dart';
import 'package:twipe_flutter_core/utils/controller/collection_controller.dart';
import 'package:twipe_flutter_core/utils/controller/controller_manager.dart';
import 'package:twipe_flutter_core/utils/eloquent/model/model.dart';
import 'package:twipe_flutter_core/utils/test/tester.dart';

class ModelTester extends Tester {
  static const String collectionId = 'model_tester_collection';

  @override
  Future<bool> setup() async {
    await ControllerManager.registerCollectionController(
        new _TestCollection(collectionId));
    for (int i = 0; i < 10; i++) {
      Model model = ControllerManager.getCollectionController(collectionId)
          .createModel({'id': 'test' + i.toString()})!;
      await ControllerManager.getCollectionController(collectionId)
          .saveModel(model);
    }
    return true;
  }

  @override
  Future<bool> clear() async {
    await ControllerManager.getCollectionController(collectionId).clear();
    ControllerManager.unregisterCollectionController(collectionId);
    return true;
  }

  @override
  Map<String, dynamic> get tests => {
        'modelCanGetAll': modelCanGetAll,
        'modelCanSave': modelCanSave,
        'modelCanDelete': modelCanDelete
      };

  Future<void> modelCanGetAll() async {
    Map<String, Model> result = Model.all<_TestCollection>(collectionId);
    expect(result.length, 10);
  }

  Future<void> modelCanSave() async {
    Model model = ControllerManager.getCollectionController(collectionId)
        .createModel({'id': 'test'})!;
    bool result = await model.save();
    expect(result, true);
    expect(
        ControllerManager.getCollectionController(collectionId)
            .hasModel('test'),
        true);
  }

  Future<void> modelCanDelete() async {
    Model model = ControllerManager.getCollectionController(collectionId)
        .createModel({'id': 'test'})!;
    bool save = await model.save();
    expect(save, true);
    bool delete = await model.delete();
    expect(delete, true);
    expect(
        ControllerManager.getCollectionController(collectionId)
            .hasModel('test'),
        false);
  }
}

///HELPER

class _TestCollection extends CollectionController<_TestModel> {
  _TestCollection(String id) : super(id);

  @override
  _TestModel? createModel(Map<String, dynamic> modelData) {
    _TestModel collectionHelperModel = _TestModel(modelData);
    if (!collectionHelperModel.validate()) {
      return null;
    }
    return collectionHelperModel;
  }
}

class _TestModel extends Model {
  _TestModel(Map<String, dynamic> data) : super(data);

  @override
  List<String> get baseFields => ['id=required'];

  @override
  String get defaultCollection => ModelTester.collectionId;

  @override
  List<String> get hiddenFields => [];

  @override
  List<String> get validator => [];
}
